from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import LogoutView
from django.http import HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_protect
from .settings import api_settings


class AdminLoginView(View):
    redirect_field_name = REDIRECT_FIELD_NAME
    login_url = api_settings.ADMIN_LOGIN_URL

    def get(self, request):
        next_url = request.GET.get(self.redirect_field_name)
        if next_url:
            return HttpResponseRedirect(self.login_url + f"?next={next_url}")
        return HttpResponseRedirect(self.login_url)


class AdminLogoutView(LogoutView):

    @method_decorator(csrf_protect)
    def post(self, request, *args, **kwargs):
        """
        Does absolutely nothing to log the user out.
        Not meant to be used.
        """
        redirect_to = self.get_success_url()
        if redirect_to != request.get_full_path():
            response = HttpResponseRedirect(redirect_to)
        else:
            response = super().get(request, *args, **kwargs)
        return response



from drf_spectacular.contrib.rest_auth import SimpleJWTCookieScheme


class SimpleJWTTokenUserScheme(SimpleJWTCookieScheme):
    target_class = "email_auth_remote.authentication.JWTStatelessCookieAuthentication"



from rest_framework_simplejwt.exceptions import (
    InvalidToken,
    AuthenticationFailed,
)

from .authentication import JWTStatelessCookieAuthentication


class JWTStatelessCookieAuthMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        authentication = JWTStatelessCookieAuthentication()

        try:
            result = authentication.authenticate(request)
            if result is not None:
                request.user = result[0]
        except (InvalidToken, AuthenticationFailed):
            pass

        return self.get_response(request)



"""Модуль настройки приложения email auth remote"""

from django.apps import AppConfig


class EmailAuthRemoteConfig(AppConfig):
    """Настройка приложения email auth remote"""

    default_auto_field = "django.db.models.BigAutoField"
    name = "email_auth_remote"



from .settings import api_settings


def constants(request):
    return {
        "JWT_ACCESS_CHECK_URL": api_settings.JWT_ACCESS_CHECK_URL,
        "JWT_REFRESH_URL": api_settings.JWT_REFRESH_URL,
        "LOGOUT_URL": api_settings.LOGOUT_URL,
    }



from django.conf import settings
from rest_framework.settings import APISettings


USER_SETTINGS = getattr(settings, "EMAIL_AUTH_REMOTE", None)

DEFAULTS = {
    "JWT_REFRESH_URL": None,
    "JWT_ACCESS_CHECK_URL": None,
    "ADMIN_LOGIN_URL": None,
    "LOGOUT_URL": None,
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = ()

api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)



from rest_framework import exceptions
from rest_framework.authentication import CSRFCheck

from .settings import api_settings


def enforce_csrf(request):
    """
    Enforce CSRF validation for session based authentication.
    """

    def dummy_get_response(request):  # pragma: no cover
        return None

    check = CSRFCheck(dummy_get_response)
    # populates request.META['CSRF_COOKIE'], which is used in process_view()
    check.process_request(request)
    reason = check.process_view(request, None, (), {})
    if reason:
        # CSRF failed, bail with explicit error message
        raise exceptions.PermissionDenied(f"CSRF Failed: {reason}")



"""Модуль аутентификации с помощью auth endpoint."""

import logging
from rest_framework_simplejwt.authentication import AuthUser
from rest_framework_simplejwt.settings import api_settings as jwt_settings
from rest_framework_simplejwt.exceptions import InvalidToken
from rest_framework_simplejwt.tokens import Token
from dj_rest_auth.jwt_auth import JWTCookieAuthentication

logger = logging.getLogger(__name__)


class JWTStatelessCookieAuthentication(JWTCookieAuthentication):
    """
    An authentication plugin that hopefully authenticates requests through a JSON web
    token provided in a request cookie (and through the header as normal, with a
    preference to the header).
    """

    def get_user(self, validated_token: Token) -> AuthUser:
        """
        Returns a stateless user object which is backed by the given validated
        token.
        """
        if jwt_settings.USER_ID_CLAIM not in validated_token:
            # The TokenUser class assumes tokens will have a recognizable user
            # identifier claim.
            raise InvalidToken(_("Token contained no recognizable user identification"))

        return jwt_settings.TOKEN_USER_CLASS(validated_token)
